#!/usr/bin/env python3

import click

import mucus.command
import mucus.config
import mucus.deezer.client
import mucus.exception
import mucus.history
import mucus.play


@click.command()
@click.option('--alias', '-a', 'aliases', type=(str, str), multiple=True)
def command(aliases):
    aliases = dict(aliases)
    config = mucus.config.Config()

    with mucus.history.History(__name__) as history:
        client = mucus.deezer.client.Client()
        player = mucus.play.Player(client)
        player.start()

        def inputs():
            while True:
                try:
                    yield input('> ')
                except EOFError:
                    break

        for line in inputs():
            if line.strip() == '':
                continue

            try:
                loader = mucus.command.Loader(line=line, aliases=aliases)
            except mucus.command.NoSuchCommand:
                loader = mucus.command.Loader(name='search')

            runner = mucus.command.Runner(
                loader,
                context={
                    'client': client,
                    'command': {
                        'line': line,
                        'name': loader.name
                    },
                    'config': config,
                    'history': history,
                    'player': player
                }
            )

            try:
                runner()
            except mucus.exception.ExitException:
                break
            except Exception as e:
                raise click.ClickException(e)


def main():
    return command(default_map=mucus.config.Config().flatten())



import queue
import subprocess
import threading
import time

import mucus.song


class Player:
    def __init__(self, client):
        self.client = client
        self.queue = queue.Queue()
        self.events = {name: threading.Event() for name in ('pause', 'stop')}
        self.thread = None
        self.state = {}

    def play(self, song):
        data = self.client.stream(song)

        media = next(data)
        if media is None:
            raise Exception

        source = next(data)
        if source is None:
            raise Exception

        song = mucus.song.Song(data=song)

        self.state.update({'song': song})

        with subprocess.Popen(['sox', '-', '-d', '-q', '-V0'], stdin=subprocess.PIPE) as p: # noqa
            for chunk in data:
                if self.events['stop'].is_set():
                    break
                while self.events['pause'].is_set():
                    time.sleep(0.1)
                p.stdin.write(chunk)

        self.state.update({'last_song': song, 'song': None})

    def loop(self):
        self.events['stop'].clear()
        self.state.update({'state': 'running'})
        while not self.events['stop'].is_set():
            try:
                song = self.queue.get(timeout=1)
            except queue.Empty:
                continue
            if song is None:
                break
            self.play(song)
            self.queue.task_done()
        self.state.update({'state': 'stopped'})

    def start(self):
        self.thread = threading.Thread(target=self.loop, daemon=True)
        self.thread.start()



import httpx
import redis

from mucus.deezer.exception import AuthException


def discover(key):
    with redis.Redis(decode_responses=True) as db:
        value = db.get(f'mucus:deezer:{key}')
        if value is not None:
            return value
    raise AuthException(key)


class Auth(httpx.Auth):
    def __init__(self, sid=None, arl=None):
        if sid is None:
            sid = discover('sid')
        if arl is None:
            arl = discover('arl')
        self.sid = sid
        self.arl = arl

    def auth_flow(self, request):
        request.cookies['sid'] = self.sid
        request.cookies['arl'] = self.arl
        yield request



import httpx

from mucus.deezer.decrypter import Decrypter


class Stream:
    def __init__(self, song, license_token):
        try:
            song = song.data
        except AttributeError:
            pass
        self.song = song
        self.license_token = license_token

    def stream(self, format='FLAC'):
        media = httpx.post('https://media.deezer.com/v1/get_url', json={
            'license_token': self.license_token,
            'media': [{'type': 'FULL',
                       'formats': [{'cipher': 'BF_CBC_STRIPE',
                                    'format': format}]}],
            'track_tokens': [self.song['TRACK_TOKEN']]
        }).json()['data'][0]['media'][0]
        yield media
        decrypter = Decrypter(self.song['SNG_ID'])
        for source in media['sources']:
            with httpx.stream('GET', source['url']) as r:
                yield source
                for i, chunk in enumerate(r.iter_bytes(chunk_size=2048)):
                    if i % 3 == 0 and len(chunk) == 2048:
                        chunk = decrypter.decrypt(chunk)
                    yield chunk
                return



import mucus.command


class Command(mucus.command.Command):
    def __call__(self, player, **kwargs):
        pause = player.events['pause']
        if pause.is_set():
            pause.clear()
        else:
            pause.set()



import hashlib

import Crypto.Cipher.Blowfish


class Decrypter:
    def __init__(self, song_id, secret=b'g4el58wc0zvf9na1', iv=bytes(range(8))): # noqa
        h = hashlib.md5(song_id.encode()).hexdigest().encode()
        self.key = bytes([h[i] ^ h[i+16] ^ secret[i] for i in range(16)])
        self.iv = iv

    def decrypt(self, chunk):
        return Crypto.Cipher.Blowfish.new(self.key, Crypto.Cipher.Blowfish.MODE_CBC, self.iv).decrypt(chunk) # noqa
